{% load news %}
{% if category_list %}
    <section class="news-category-list">
        <h1>Categories</h1>
        <ul>
            <li>
                <a{% if not current_category %} class="here"{% endif %} href="{% article_archive_url %}">All categories</a>
            </li>
            {% for category in category_list %}
                <li>
                    <a{% if current_category == category %} class="here"{% endif %} href="{% category_url category %}">{{ category }}</a>
                </li>
            {% endfor %}
        </ul>
    </section>
{% endif %}
