{% load thumbnail html news %}
<article{% if article.image %} class="news-article-list-item-with-image"{% endif %}>
    {% if article.image %}
        <aside>
            <a href="{% article_url article %}">
                {% thumbnail item.image "x140" crop="center" as im %}
                    <img src="{{ im.url }}" width="{{ im.width }}" height="{{ im.height }}">
                {% endthumbnail %}
            </a>
        </aside>
    {% endif %}

    <hgroup>
        <h1><a href="{% article_url article %}">{{ article }}</a></h1>
        <h2>{% article_meta article %}</h2>
    </hgroup>

    {% if article.summary %}
        {{ article.summary|html }}
    {% else %}
        <p>{{ article.content|truncate_paragraphs:1|striptags }}</p>
    {% endif %}
</article>
